/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 *                                                                          *
 *   Credits to:                                                            *
 *   Christian "quadratqualle" Lutze for code basics                        *
 *   Dario Carluccio for TID-Communication Description                      *
 ****************************************************************************/

#include "C-TID.h"

uint32_t ticks = 0;
uint8_t timer0_offset = 0;

/**
 * visualize the Rounds per Minute as void function to pass it to the scrolltext
 */
void visualizeRPM(void) {
    TID_symbol_bargraph(TID_symbols, 0, 6000, rpm);
}

/**
 * Main Function
 * @return 1 in case of error, should never return anyways
 */
int main(void) {

    // all symbols on
    TID_symbols[0] = TS_COMMA | TS_RDS | TS_TP | TS_STEREO | TS_AS | TS_TPBRACKETS;
    TID_symbols[1] = TS_CD | TS_DOLBYB | TS_DOLBYC | TS_CR | TS_CPS;

    // initialize timers and capture pins
    timer0_init();
    timer1_init();
    int0_init();
    icp_init();

    // enable global interrupts
    sei();

    // reset the display harder...
    DISPLAY_PIN_OUTPUT
    DISPLAY_OFF
    _delay_ms(500);
    DISPLAY_ON
    _delay_ms(50);
    DISPLAY_OFF
    _delay_ms(50);
    DISPLAY_ON

    TID_symbols[0] &= ~(TS_COMMA); // no comma
    TID_scrolltext(PSTR("ANSCHNALLEN UND AB GEHT DIE POST"), TID_symbols, 1, &visualizeRPM);

    uint16_t old = uptime;
    uint16_t duration = 0;
    uint8_t state = 0;

    while (1) {
        visualizeRPM();

        // since free-running during digital I/O is pointless we use it as delay :)
        volt = getVoltage(3);

        switch (state) {
            case 0:
                TID_decimal(volt, "VOLT", TID_symbols, 1);
                duration = 5;
                break;
            case 1:
				icp_disable();
				int0_enable();
                TID_decimal(rpm, "RPM", TID_symbols, 0);
                duration = 5;
                break;
            case 2:
				int0_disable();
				icp_enable();
                TID_decimal(speed, "KMH", TID_symbols, 1);
                duration = 5;
                break;
            case 3:
                TID_symbols[0] &= ~(TS_COMMA); // no comma
                TID_scrolltext(PSTR("2 SEC DATUM"), TID_symbols, 1, &visualizeRPM);
                duration = 0; // only one time
                break;
            case 4:
                DISPLAY_OFF;
                duration = 2; // 2sec date is enough
                break;
            default:
                state = 0;
        }

        if ((uptime > old ? (uint16_t)(uptime - old) : (uint16_t)(old - uptime)) > duration) {
            old = uptime;
            state++;
            DISPLAY_ON
        }

        _delay_ms(50);
    }


    return 1;
}

