/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 ****************************************************************************/

#include "analog.h"


#define CHANNEL 3

#define prim_r1 1
#define prim_r2 10

/// global variable for voltage measured
volatile int16_t volt;



/*



// free running is not good during digital I/O

void analog_init() {
    // set up channel
    ADMUX = CHANNEL;

    // VCC used as analog reference, disconnected from PA0 (AREF)
    ADMUX &= ~((1 << REFS1) | (1 << REFS0));
    // External voltage reference at PA0 (AREF) pin, internal reference turned off
    //ADMUX &= ~(1 << REFS1);
    //ADMUX |= (1 << REFS0);
    // Internal 1.1V voltage reference
    //ADMUX |= (1 << REFS1) | (0 << REFS0);

    // Free Running mode
    ADCSRB = ~(( 1 << ADTS2 ) | ( 1 << ADTS1 ) | ( 1 << ADTS0 ));

    // enable ADC and set clk div to 128
    ADCSRB |= ((1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0));

    // ADC Auto Trigger Enable | ADC Interrupt Enable
    //ADCSRA |= ((1 << ADATE) | (1 << ADIE));

    // ADC Interrupt Enable
    ADCSRA |= (1 << ADIE);

    // start ADC
    ADCSRA |= (1 << ADSC);
}

#if defined (__AVR_ATtiny84__)
ISR (ADC_vect) {
#else
ISR (ADC_vect) {
#endif
            static uint8_t times = 0;
            static uint16_t result = 0;

            result += ADCW;

            if (times++ == 3) {
                static int16_t raw_volt = 0;
                raw_volt = result / 3;
                raw_volt = ((uint32_t)raw_volt * (uint32_t)48828) / (uint32_t)10000;
                volt = (int16_t) (((uint32_t)raw_volt * (uint32_t)(prim_r1 + prim_r2)) / (uint32_t)prim_r1) / 100;
                times = 0;
                result = 0;
            }

            // please try again
            ADCSRA |= (1 << ADSC);
}
 */

/**
 * Read out the ADC and compute voltage from it
 * @param channel the ADC channel to read
 * @return measured voltage in 10th of volts
 */
uint16_t readADC(uint8_t channel) {
    uint8_t i;
    uint16_t result = 0;

    // enable ADC and set clk div to 64
    //ADCSRA = ((1<<ADEN) | (1<<ADPS2) | (1<<ADPS1));
    // enable ADC and set clk div to 32
    //ADCSRA = ((1<<ADEN) | (1<<ADPS2) | (1<<ADPS0));
    // enable ADC and set clk div to 16
    ADCSRA = ((1 << ADEN) | (1 << ADPS2));


    _delay_us(1);

    // set up channel
    ADMUX = channel;
    // use internal reference
    //ADMUX |= (1<<REFS1) | (1<<REFS0);

    // init ADC for a dummy readout
    ADCSRA |= (1 << ADSC);
    // wait for conversion to be complete
    while (ADCSRA & (1 << ADSC));

    // read in three times and get the average
    for (i = 0; i < 3; i++) {
        // start conversion
        ADCSRA |= (1 << ADSC);

        // wait for conversion to be complete
        while (ADCSRA & (1 << ADSC));

        // add up result
        result += ADCW;
    }

    // disable ADC
    ADCSRA &= ~(1 << ADEN);

    // get average
    result /= 3;

    return result;
}

uint8_t getVoltage(uint8_t channel) {
    static int16_t raw_volt = 0;
    raw_volt = readADC(channel);
    /* according to what i read about voltage divider it is
       Uo = Ue * (R1 / (R2 + R1))
       Ue = Uo * (R2 + R1) / R1
       the board has got r1 = 4.7k and r2 = 15k
       but since 1step is 0,0048828125V = 4,8828125mV and not 5mV there
       is some conversion to do for raw_volt --**-> Uo
       this should end up in 10th of volts */
    raw_volt = ((uint32_t)raw_volt * (uint32_t)48828) / (uint32_t)10000;
    static int16_t retvolt = 0;
    retvolt = (int16_t)(((uint32_t)raw_volt * (uint32_t)(prim_r1 + prim_r2)) / (uint32_t)prim_r1) / 100;
    if (volt < 0) retvolt = 0;

    return retvolt;
}
