/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 ****************************************************************************/
 
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#ifndef _ICP_H
#define _ICP_H

// in cm
#define WHEEL_CIRCUMFERENCE	177
// opel sends 32 signals per wheel turn
#define SPD_SIGNALS_PER_TURN 32

/*
 * example:
 *
 * WHEEL_CIRCUMFERENCE (145/80 R 13) = 1766,2mm ^= 177cm
 * => 100 000 / 177 = 564.971751 = > 565 turns / km
 * SIGNALS_PER_TURN = 32
 * => 18080 signals / km
 *
 * speed in km/h || time between signals in s
 *       1       || 0,199.115.044
 *      30       || 0,006.637.168.14
 *      50       || 0,003.982.300.88
 *     100       || 0,001.991.150.44
 *
 * => longest time to wait for a valid speed signal = ~200ms
 */
#define MAX_VALID_SPEED_TIME 200


void icp_init();

void icp_enable();

void icp_disable();

extern volatile uint16_t speed;

#if defined (__AVR_ATtiny84__)	
//ISR (PCINT0_vect) {
ISR(TIM1_CAPT_vect);
#else
ISR(PCINT0_vect);
#endif


#endif
