/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 *                                                                          *
 *   Credits to:                                                            *
 *   Christian "quadratqualle" Lutze for code basics                        *
 *   Dario Carluccio for TID-Communication Description                      *
 ****************************************************************************/
 
#include <avr/io.h>
#include <util/delay.h>
#include <util/parity.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <string.h>

#ifndef _OPEL_TID_H
#define _OPEL_TID_H

// 8 or 10 characters
#define TID_TYPE 8

// original timings were too short
#define WAIT_FACTOR * 2


#if defined (__AVR_ATtiny84__)

#define TID_PORT	PORTA
#define TID_DDR		DDRA
#define TID_PIN		PINA

#define TID_MRQ		5
#define	TID_SDA		6
#define TID_SCL		4

#else

#define TID_PORT	PORTA
#define TID_DDR		DDRA
#define TID_PIN		PINA

#define TID_MRQ		3
#define	TID_SDA		4
#define TID_SCL		5

#endif


#define TS_COMMA (1 << 6)
#define TS_RDS (1 << 5)
#define TS_TP (1 << 4)
#define TS_STEREO (1 << 3)
#define TS_AS (1 << 1)
#define TS_TPBRACKETS (1 << 0)


#define TS_CD (1 << 6)
#define TS_DOLBYB (1 << 5)
#define TS_DOLBYC (1 << 4)
#define TS_CR (1 << 3)
#define TS_CPS (1 << 2)

#define TS_TRACK (1 << 5)
#define TS_RDM (1 << 4)
#define TS_PGM (1 << 3)
#define TS_DISC (1 << 2)


/*
                Symbol-Bytes	Daten-Bytes	Slave Adresse
8-stelliges TID	 	2	 8		0x4a (74 dec)
10-stelliges TID	3	 10		0x4d (77 dec)
 */
#if TID_TYPE == 10
#define TID_address	0x4D // 10 stellig
#define TID_characters 10
unsigned char TID_symbols[3]; // = {0, 0, 0};
#else
#define TID_address	0x4A // 8stellig
#define TID_characters 8
unsigned char TID_symbols[2]; // = {0, 0};
#endif

#define SCROLLTEXTDELAY 10


#ifndef cbi
#define cbi(sfr, bit)     (_SFR_BYTE(sfr) &= ~_BV(bit)) 
#endif
#ifndef sbi
#define sbi(sfr, bit)     (_SFR_BYTE(sfr) |= _BV(bit))  
#endif



void TID_puts(char buffer[], unsigned char symbols[]);

void TID_symbol_bargraph(unsigned char symbols[], int16_t min, int16_t max, int16_t value);

void TID_scrolltext(char text[], unsigned char symbols[], uint8_t pgm, void (*functionP)(void));

void TID_decimal(int16_t number, char description[], unsigned char symbols[], int scale);

#endif // #ifndef _OPEL_TID_H
